package jdk.internal.org.objectweb.asm.commons;

import java.util.Arrays;
import jdk.internal.org.objectweb.asm.Label;
import jdk.internal.org.objectweb.asm.Handle;
import jdk.internal.org.objectweb.asm.ClassVisitor;
import java.util.ArrayList;
import jdk.internal.org.objectweb.asm.MethodVisitor;
import java.util.List;
import jdk.internal.org.objectweb.asm.Type;

public class GeneratorAdapter extends LocalVariablesSorter
{
    private static final String CLDESC = "Ljava/lang/Class;";
    private static final Type BYTE_TYPE;
    private static final Type BOOLEAN_TYPE;
    private static final Type SHORT_TYPE;
    private static final Type CHARACTER_TYPE;
    private static final Type INTEGER_TYPE;
    private static final Type FLOAT_TYPE;
    private static final Type LONG_TYPE;
    private static final Type DOUBLE_TYPE;
    private static final Type NUMBER_TYPE;
    private static final Type OBJECT_TYPE;
    private static final Method BOOLEAN_VALUE;
    private static final Method CHAR_VALUE;
    private static final Method INT_VALUE;
    private static final Method FLOAT_VALUE;
    private static final Method LONG_VALUE;
    private static final Method DOUBLE_VALUE;
    public static final int ADD = 96;
    public static final int SUB = 100;
    public static final int MUL = 104;
    public static final int DIV = 108;
    public static final int REM = 112;
    public static final int NEG = 116;
    public static final int SHL = 120;
    public static final int SHR = 122;
    public static final int USHR = 124;
    public static final int AND = 126;
    public static final int OR = 128;
    public static final int XOR = 130;
    public static final int EQ = 153;
    public static final int NE = 154;
    public static final int LT = 155;
    public static final int GE = 156;
    public static final int GT = 157;
    public static final int LE = 158;
    private final int access;
    private final Type returnType;
    private final Type[] argumentTypes;
    private final List<Type> localTypes;
    
    public GeneratorAdapter(MethodVisitor methodVisitor, int n, String s, String s2) {
        this(327680, methodVisitor, n, s, s2);
        if (this.getClass() != GeneratorAdapter.class) {
            throw new IllegalStateException();
        }
        return;
    }
    
    protected GeneratorAdapter(int n, MethodVisitor methodVisitor, int access, String s, String s2) {
        super(n, access, s2, methodVisitor);
        this.localTypes = new ArrayList();
        this.access = access;
        this.returnType = Type.getReturnType(s2);
        this.argumentTypes = Type.getArgumentTypes(s2);
    }
    
    public GeneratorAdapter(int n, Method method, MethodVisitor methodVisitor) {
        this(methodVisitor, n, null, method.getDescriptor());
    }
    
    public GeneratorAdapter(int access, Method method, String signature, Type[] array, ClassVisitor classVisitor) {
        this(access, method, classVisitor.visitMethod(access, method.getName(), method.getDescriptor(), signature, getInternalNames(array)));
    }
    
    private static String[] getInternalNames(Type[] array) {
        if (array == null) {
            return null;
        }
        String[] array2 = new String[array.length];
        for (int i = 0; i < array2.length; ++i) {
            array2[i] = array[i].getInternalName();
        }
        return array2;
    }
    
    public void push(boolean b) {
        this.push(b ? 1 : 0);
    }
    
    public void push(int operand) {
        if (operand >= -1 && operand <= 5) {
            this.mv.visitInsn(3 + operand);
        }
        else if (operand >= -128 && operand <= 127) {
            this.mv.visitIntInsn(16, operand);
        }
        else if (operand >= -32768 && operand <= 32767) {
            this.mv.visitIntInsn(17, operand);
        }
        else {
            this.mv.visitLdcInsn(Integer.valueOf(operand));
        }
    }
    
    public void push(long l) {
        if (l == 0L || l == 1L) {
            this.mv.visitInsn(9 + (int)l);
        }
        else {
            this.mv.visitLdcInsn(Long.valueOf(l));
        }
    }
    
    public void push(float n) {
        int floatToIntBits = Float.floatToIntBits(n);
        if ((long)floatToIntBits == 0L || floatToIntBits == 1065353216 || floatToIntBits == 1073741824) {
            this.mv.visitInsn(11 + (int)n);
        }
        else {
            this.mv.visitLdcInsn(Float.valueOf(n));
        }
    }
    
    public void push(double n) {
        long doubleToLongBits = Double.doubleToLongBits(n);
        if (doubleToLongBits == 0L || doubleToLongBits == 4607182418800017408L) {
            this.mv.visitInsn(14 + (int)n);
        }
        else {
            this.mv.visitLdcInsn(Double.valueOf(n));
        }
    }
    
    public void push(String value) {
        if (value == null) {
            this.mv.visitInsn(1);
        }
        else {
            this.mv.visitLdcInsn(value);
        }
    }
    
    public void push(Type value) {
        if (value == null) {
            this.mv.visitInsn(1);
        }
        else {
            switch (value.getSort()) {
                case 1: {
                    this.mv.visitFieldInsn(178, "java/lang/Boolean", "TYPE", "Ljava/lang/Class;");
                    break;
                }
                case 2: {
                    this.mv.visitFieldInsn(178, "java/lang/Character", "TYPE", "Ljava/lang/Class;");
                    break;
                }
                case 3: {
                    this.mv.visitFieldInsn(178, "java/lang/Byte", "TYPE", "Ljava/lang/Class;");
                    break;
                }
                case 4: {
                    this.mv.visitFieldInsn(178, "java/lang/Short", "TYPE", "Ljava/lang/Class;");
                    break;
                }
                case 5: {
                    this.mv.visitFieldInsn(178, "java/lang/Integer", "TYPE", "Ljava/lang/Class;");
                    break;
                }
                case 6: {
                    this.mv.visitFieldInsn(178, "java/lang/Float", "TYPE", "Ljava/lang/Class;");
                    break;
                }
                case 7: {
                    this.mv.visitFieldInsn(178, "java/lang/Long", "TYPE", "Ljava/lang/Class;");
                    break;
                }
                case 8: {
                    this.mv.visitFieldInsn(178, "java/lang/Double", "TYPE", "Ljava/lang/Class;");
                    break;
                }
                default: {
                    this.mv.visitLdcInsn(value);
                    break;
                }
            }
        }
    }
    
    public void push(Handle value) {
        this.mv.visitLdcInsn(value);
    }
    
    private int getArgIndex(int n) {
        int n2 = (this.access & 0x8) == 0x0;
        for (int i = 0; i < n; ++i) {
            n2 += this.argumentTypes[i].getSize();
        }
        return n2;
    }
    
    private void loadInsn(Type type, int varIndex) {
        this.mv.visitVarInsn(type.getOpcode(21), varIndex);
    }
    
    private void storeInsn(Type type, int varIndex) {
        this.mv.visitVarInsn(type.getOpcode(54), varIndex);
    }
    
    public void loadThis() {
        if ((this.access & 0x8) != 0x0) {
            throw new IllegalStateException("no 'this' pointer within static method");
        }
        this.mv.visitVarInsn(25, 0);
        return;
    }
    
    public void loadArg(int n) {
        this.loadInsn(this.argumentTypes[n], this.getArgIndex(n));
    }
    
    public void loadArgs(int n, int n2) {
        Type type;
        for (int argIndex = this.getArgIndex(n), i = 0; i < n2; argIndex += type.getSize(), ++i) {
            type = this.argumentTypes[n + i];
            this.loadInsn(type, argIndex);
        }
    }
    
    public void loadArgs() {
        this.loadArgs(0, this.argumentTypes.length);
    }
    
    public void loadArgArray() {
        this.push(this.argumentTypes.length);
        this.newArray(GeneratorAdapter.OBJECT_TYPE);
        for (int i = 0; i < this.argumentTypes.length; ++i) {
            this.dup();
            this.push(i);
            this.loadArg(i);
            this.box(this.argumentTypes[i]);
            this.arrayStore(GeneratorAdapter.OBJECT_TYPE);
        }
    }
    
    public void storeArg(int n) {
        this.storeInsn(this.argumentTypes[n], this.getArgIndex(n));
    }
    
    public Type getLocalType(int n) {
        return (Type)this.localTypes.get(n - this.firstLocal);
    }
    
    @Override
    protected void setLocalType(int n, Type type) {
        int n2 = n - this.firstLocal;
        while (this.localTypes.size() < n2 + 1) {
            this.localTypes.add(null);
        }
        this.localTypes.set(n2, type);
    }
    
    public void loadLocal(int n) {
        this.loadInsn(this.getLocalType(n), n);
    }
    
    public void loadLocal(int n, Type type) {
        this.setLocalType(n, type);
        this.loadInsn(type, n);
    }
    
    public void storeLocal(int n) {
        this.storeInsn(this.getLocalType(n), n);
    }
    
    public void storeLocal(int n, Type type) {
        this.setLocalType(n, type);
        this.storeInsn(type, n);
    }
    
    public void arrayLoad(Type type) {
        this.mv.visitInsn(type.getOpcode(46));
    }
    
    public void arrayStore(Type type) {
        this.mv.visitInsn(type.getOpcode(79));
    }
    
    public void pop() {
        this.mv.visitInsn(87);
    }
    
    public void pop2() {
        this.mv.visitInsn(88);
    }
    
    public void dup() {
        this.mv.visitInsn(89);
    }
    
    public void dup2() {
        this.mv.visitInsn(92);
    }
    
    public void dupX1() {
        this.mv.visitInsn(90);
    }
    
    public void dupX2() {
        this.mv.visitInsn(91);
    }
    
    public void dup2X1() {
        this.mv.visitInsn(93);
    }
    
    public void dup2X2() {
        this.mv.visitInsn(94);
    }
    
    public void swap() {
        this.mv.visitInsn(95);
    }
    
    public void swap(Type type, Type type2) {
        if (type2.getSize() == 1) {
            if (type.getSize() == 1) {
                this.swap();
            }
            else {
                this.dupX2();
                this.pop();
            }
        }
        else if (type.getSize() == 1) {
            this.dup2X1();
            this.pop2();
        }
        else {
            this.dup2X2();
            this.pop2();
        }
    }
    
    public void math(int opcode, Type type) {
        this.mv.visitInsn(type.getOpcode(opcode));
    }
    
    public void not() {
        this.mv.visitInsn(4);
        this.mv.visitInsn(130);
    }
    
    public void iinc(int varIndex, int increment) {
        this.mv.visitIincInsn(varIndex, increment);
    }
    
    public void cast(Type type, Type type2) {
        if (type != type2) {
            if (type == Type.DOUBLE_TYPE) {
                if (type2 == Type.FLOAT_TYPE) {
                    this.mv.visitInsn(144);
                }
                else if (type2 == Type.LONG_TYPE) {
                    this.mv.visitInsn(143);
                }
                else {
                    this.mv.visitInsn(142);
                    this.cast(Type.INT_TYPE, type2);
                }
            }
            else if (type == Type.FLOAT_TYPE) {
                if (type2 == Type.DOUBLE_TYPE) {
                    this.mv.visitInsn(141);
                }
                else if (type2 == Type.LONG_TYPE) {
                    this.mv.visitInsn(140);
                }
                else {
                    this.mv.visitInsn(139);
                    this.cast(Type.INT_TYPE, type2);
                }
            }
            else if (type == Type.LONG_TYPE) {
                if (type2 == Type.DOUBLE_TYPE) {
                    this.mv.visitInsn(138);
                }
                else if (type2 == Type.FLOAT_TYPE) {
                    this.mv.visitInsn(137);
                }
                else {
                    this.mv.visitInsn(136);
                    this.cast(Type.INT_TYPE, type2);
                }
            }
            else if (type2 == Type.BYTE_TYPE) {
                this.mv.visitInsn(145);
            }
            else if (type2 == Type.CHAR_TYPE) {
                this.mv.visitInsn(146);
            }
            else if (type2 == Type.DOUBLE_TYPE) {
                this.mv.visitInsn(135);
            }
            else if (type2 == Type.FLOAT_TYPE) {
                this.mv.visitInsn(134);
            }
            else if (type2 == Type.LONG_TYPE) {
                this.mv.visitInsn(133);
            }
            else if (type2 == Type.SHORT_TYPE) {
                this.mv.visitInsn(147);
            }
        }
    }
    
    private static Type getBoxedType(Type type) {
        switch (type.getSort()) {
            case 3: {
                return GeneratorAdapter.BYTE_TYPE;
            }
            case 1: {
                return GeneratorAdapter.BOOLEAN_TYPE;
            }
            case 4: {
                return GeneratorAdapter.SHORT_TYPE;
            }
            case 2: {
                return GeneratorAdapter.CHARACTER_TYPE;
            }
            case 5: {
                return GeneratorAdapter.INTEGER_TYPE;
            }
            case 6: {
                return GeneratorAdapter.FLOAT_TYPE;
            }
            case 7: {
                return GeneratorAdapter.LONG_TYPE;
            }
            case 8: {
                return GeneratorAdapter.DOUBLE_TYPE;
            }
            default: {
                return type;
            }
        }
    }
    
    public void box(Type type) {
        if (type.getSort() == 10 || type.getSort() == 9) {
            return;
        }
        if (type == Type.VOID_TYPE) {
            this.push((String)null);
        }
        else {
            Type boxedType = getBoxedType(type);
            this.newInstance(boxedType);
            if (type.getSize() == 2) {
                this.dupX2();
                this.dupX2();
                this.pop();
            }
            else {
                this.dupX1();
                this.swap();
            }
            this.invokeConstructor(boxedType, new Method("<init>", Type.VOID_TYPE, new Type[] { type }));
        }
        return;
    }
    
    public void valueOf(Type type) {
        if (type.getSort() == 10 || type.getSort() == 9) {
            return;
        }
        if (type == Type.VOID_TYPE) {
            this.push((String)null);
        }
        else {
            Type boxedType;
            this.invokeStatic(boxedType, new Method("valueOf", boxedType = getBoxedType(type), new Type[] { type }));
        }
        return;
    }
    
    public void unbox(Type type) {
        Type type2 = GeneratorAdapter.NUMBER_TYPE;
        Method method = null;
        switch (type.getSort()) {
            case 0: {
                return;
            }
            case 2: {
                type2 = GeneratorAdapter.CHARACTER_TYPE;
                method = GeneratorAdapter.CHAR_VALUE;
                break;
            }
            case 1: {
                type2 = GeneratorAdapter.BOOLEAN_TYPE;
                method = GeneratorAdapter.BOOLEAN_VALUE;
                break;
            }
            case 8: {
                method = GeneratorAdapter.DOUBLE_VALUE;
                break;
            }
            case 6: {
                method = GeneratorAdapter.FLOAT_VALUE;
                break;
            }
            case 7: {
                method = GeneratorAdapter.LONG_VALUE;
                break;
            }
            case 3:
            case 4:
            case 5: {
                method = GeneratorAdapter.INT_VALUE;
                break;
            }
        }
        if (method == null) {
            this.checkCast(type);
        }
        else {
            this.checkCast(type2);
            this.invokeVirtual(type2, method);
        }
    }
    
    public Label newLabel() {
        return new Label();
    }
    
    public void mark(Label label) {
        this.mv.visitLabel(label);
    }
    
    public Label mark() {
        Label label = new Label();
        this.mv.visitLabel(label);
        return label;
    }
    
    public void ifCmp(Type obj, int opcode, Label label) {
        switch (obj.getSort()) {
            case 7: {
                this.mv.visitInsn(148);
                break;
            }
            case 8: {
                this.mv.visitInsn((opcode == 156 || opcode == 157) ? 151 : 152);
                break;
            }
            case 6: {
                this.mv.visitInsn((opcode == 156 || opcode == 157) ? 149 : 150);
                break;
            }
            case 9:
            case 10: {
                switch (opcode) {
                    case 153: {
                        this.mv.visitJumpInsn(165, label);
                        return;
                    }
                    case 154: {
                        this.mv.visitJumpInsn(166, label);
                        return;
                    }
                    default: {
                        throw new IllegalArgumentException("Bad comparison for type " + obj);
                    }
                }
            }
            default: {
                int opcode2 = -1;
                switch (opcode) {
                    case 153: {
                        opcode2 = 159;
                        break;
                    }
                    case 154: {
                        opcode2 = 160;
                        break;
                    }
                    case 156: {
                        opcode2 = 162;
                        break;
                    }
                    case 155: {
                        opcode2 = 161;
                        break;
                    }
                    case 158: {
                        opcode2 = 164;
                        break;
                    }
                    case 157: {
                        opcode2 = 163;
                        break;
                    }
                }
                this.mv.visitJumpInsn(opcode2, label);
                return;
            }
        }
        this.mv.visitJumpInsn(opcode, label);
    }
    
    public void ifICmp(int n, Label label) {
        this.ifCmp(Type.INT_TYPE, n, label);
    }
    
    public void ifZCmp(int opcode, Label label) {
        this.mv.visitJumpInsn(opcode, label);
    }
    
    public void ifNull(Label label) {
        this.mv.visitJumpInsn(198, label);
    }
    
    public void ifNonNull(Label label) {
        this.mv.visitJumpInsn(199, label);
    }
    
    public void goTo(Label label) {
        this.mv.visitJumpInsn(167, label);
    }
    
    public void ret(int varIndex) {
        this.mv.visitVarInsn(169, varIndex);
    }
    
    public void tableSwitch(int[] array, TableSwitchGenerator tableSwitchGenerator) {
        float n;
        if (array.length == 0) {
            n = 0.0f;
        }
        else {
            n = (float)array.length / (float)(array[array.length - 1] - array[0] + 1);
        }
        this.tableSwitch(array, tableSwitchGenerator, n >= 0.5f);
    }
    
    public void tableSwitch(int[] keys, TableSwitchGenerator tableSwitchGenerator, boolean b) {
        int i = 1;
        while (i < keys.length) {
            if (keys[i] < keys[i - 1]) {
                throw new IllegalArgumentException("keys must be sorted ascending");
            }
            ++i;
        }
        Label label = this.newLabel();
        Label label2 = this.newLabel();
        if (keys.length > 0) {
            int length = keys.length;
            int min = keys[0];
            int max = keys[length - 1];
            int n = max - min + 1;
            if (b) {
                Label[] array = new Label[n];
                Arrays.fill(array, label);
                for (int l = 0; l < length; ++l) {
                    array[keys[l] - min] = this.newLabel();
                }
                this.mv.visitTableSwitchInsn(min, max, label, array);
                for (int n2 = 0; n2 < n; ++n2) {
                    Label label3 = array[n2];
                    if (label3 != label) {
                        this.mark(label3);
                        tableSwitchGenerator.generateCase(n2 + min, label2);
                    }
                }
            }
            else {
                Label[] labels = new Label[length];
                for (int j = 0; j < length; ++j) {
                    labels[j] = this.newLabel();
                }
                this.mv.visitLookupSwitchInsn(label, keys, labels);
                for (int k = 0; k < length; ++k) {
                    this.mark(labels[k]);
                    tableSwitchGenerator.generateCase(keys[k], label2);
                }
            }
        }
        this.mark(label);
        tableSwitchGenerator.generateDefault();
        this.mark(label2);
    }
    
    public void returnValue() {
        this.mv.visitInsn(this.returnType.getOpcode(172));
    }
    
    private void fieldInsn(int opcode, Type type, String name, Type type2) {
        this.mv.visitFieldInsn(opcode, type.getInternalName(), name, type2.getDescriptor());
    }
    
    public void getStatic(Type type, String s, Type type2) {
        this.fieldInsn(178, type, s, type2);
    }
    
    public void putStatic(Type type, String s, Type type2) {
        this.fieldInsn(179, type, s, type2);
    }
    
    public void getField(Type type, String s, Type type2) {
        this.fieldInsn(180, type, s, type2);
    }
    
    public void putField(Type type, String s, Type type2) {
        this.fieldInsn(181, type, s, type2);
    }
    
    private void invokeInsn(int opcode, Type type, Method method, boolean isInterface) {
        this.mv.visitMethodInsn(opcode, (type.getSort() == 9) ? type.getDescriptor() : type.getInternalName(), method.getName(), method.getDescriptor(), isInterface);
    }
    
    public void invokeVirtual(Type type, Method method) {
        this.invokeInsn(182, type, method, false);
    }
    
    public void invokeConstructor(Type type, Method method) {
        this.invokeInsn(183, type, method, false);
    }
    
    public void invokeStatic(Type type, Method method) {
        this.invokeInsn(184, type, method, false);
    }
    
    public void invokeInterface(Type type, Method method) {
        this.invokeInsn(185, type, method, true);
    }
    
    public void invokeDynamic(String name, String descriptor, Handle bootstrapMethodHandle, Object... bootstrapMethodArguments) {
        this.mv.visitInvokeDynamicInsn(name, descriptor, bootstrapMethodHandle, bootstrapMethodArguments);
    }
    
    private void typeInsn(int opcode, Type type) {
        this.mv.visitTypeInsn(opcode, type.getInternalName());
    }
    
    public void newInstance(Type type) {
        this.typeInsn(187, type);
    }
    
    public void newArray(Type type) {
        int operand;
        switch (type.getSort()) {
            case 1: {
                operand = 4;
                break;
            }
            case 2: {
                operand = 5;
                break;
            }
            case 3: {
                operand = 8;
                break;
            }
            case 4: {
                operand = 9;
                break;
            }
            case 5: {
                operand = 10;
                break;
            }
            case 6: {
                operand = 6;
                break;
            }
            case 7: {
                operand = 11;
                break;
            }
            case 8: {
                operand = 7;
                break;
            }
            default: {
                this.typeInsn(189, type);
                return;
            }
        }
        this.mv.visitIntInsn(188, operand);
    }
    
    public void arrayLength() {
        this.mv.visitInsn(190);
    }
    
    public void throwException() {
        this.mv.visitInsn(191);
    }
    
    public void throwException(Type type, String s) {
        this.newInstance(type);
        this.dup();
        this.push(s);
        this.invokeConstructor(type, Method.getMethod("void <init> (String)"));
        this.throwException();
    }
    
    public void checkCast(Type type) {
        if (!type.equals(GeneratorAdapter.OBJECT_TYPE)) {
            this.typeInsn(192, type);
        }
    }
    
    public void instanceOf(Type type) {
        this.typeInsn(193, type);
    }
    
    public void monitorEnter() {
        this.mv.visitInsn(194);
    }
    
    public void monitorExit() {
        this.mv.visitInsn(195);
    }
    
    public void endMethod() {
        if ((this.access & 0x400) == 0x0) {
            this.mv.visitMaxs(0, 0);
        }
        this.mv.visitEnd();
    }
    
    public void catchException(Label label, Label label2, Type type) {
        Label label3 = new Label();
        if (type == null) {
            this.mv.visitTryCatchBlock(label, label2, label3, null);
        }
        else {
            this.mv.visitTryCatchBlock(label, label2, label3, type.getInternalName());
        }
        this.mark(label3);
    }
    
    static {
        BYTE_TYPE = Type.getObjectType("java/lang/Byte");
        BOOLEAN_TYPE = Type.getObjectType("java/lang/Boolean");
        SHORT_TYPE = Type.getObjectType("java/lang/Short");
        CHARACTER_TYPE = Type.getObjectType("java/lang/Character");
        INTEGER_TYPE = Type.getObjectType("java/lang/Integer");
        FLOAT_TYPE = Type.getObjectType("java/lang/Float");
        LONG_TYPE = Type.getObjectType("java/lang/Long");
        DOUBLE_TYPE = Type.getObjectType("java/lang/Double");
        NUMBER_TYPE = Type.getObjectType("java/lang/Number");
        OBJECT_TYPE = Type.getObjectType("java/lang/Object");
        BOOLEAN_VALUE = Method.getMethod("boolean booleanValue()");
        CHAR_VALUE = Method.getMethod("char charValue()");
        INT_VALUE = Method.getMethod("int intValue()");
        FLOAT_VALUE = Method.getMethod("float floatValue()");
        LONG_VALUE = Method.getMethod("long longValue()");
        DOUBLE_VALUE = Method.getMethod("double doubleValue()");
    }
}
